
# TODO: Split into different classes (Like: Person, Article, WebPage etc.)
class InterchangeProperty():
    Name = "name"

    Url = "url"
    UseProxy = "useproxy"
    Html = "html"
    Text = "text"
    Title = "title"
    Subtitle = "subtitle"
    Tag = "tag"
    Author = "author"
    Credits = "credits"
    Source = "source"

    Query = "query"

    Protocol = "protocol"
    IpAddress = "ipaddress"
    Host = "host"
    Token = "token"
    Username = "username"
    Password = "password"

    Path = "path"
    Filename = "filename"

    Image = "image"
    Video = "video"

    Date = "date"
    DateTime = "datetime"
    Time = "time"

    Email = "email"
    Phone = "phone"
    Firstname = "firstname"
    Lastname = "lastname"
    Address = "address"
    ZipCode = "zipcode"
    City = "city"
    Country = "country"
    Region = "region"
    TimeZone = "timezone"
